// C Source File
// Created 9/21/2003; 12:10:58 AM

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

#define ENABLE_ERROR_RETURN   // Enable Returning Errors to AMS
#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization
#define MIN_AMS 200           // Compile for AMS 2.00 or higher
#define SAVE_SCREEN           // Save/Restore LCD Contents
#define USE_GRAY_PATCH   // use grayscale patch

#include <tigcclib.h>         // Include All Header Files
#include "images.h"

/*---------------------------------------------------------------------------*/
/* global variables                                                          */
/*---------------------------------------------------------------------------*/
int _ti92plus;          // needed to produce ti92plus code
int _ti89;              // needed to produce ti89 code

/*===========================================================================*/
/* simple wait routine                                                       */
/*===========================================================================*/
asm("xdef __waitms
__waitms: movem.l %d0-%d3,-(%sp)
		  move.w (20,%sp),%d2
		  move.l #31,%d1
		  move.l #31,%d3
_wl2_: move.w #120,%d0 /* modify this value for exact timing !!! */
_wl1_: rol.l %d3,%d1
	   dbra %d0,_wl1_
	   dbra %d2,_wl2_
	   movem.l (%sp)+,%d0-%d3
	   rts");
#define wait __waitms
	
int badger_array[12][2] =
{
	{125, 75},
	{100, 55},
	{115,  5},
	{95,  25},
	{85,  15},
	{80,  65},
	{65,  25},
	{40,  60},
	{30,  15},
	{15,  75},
	{5,   45},
	{5,   10}
};
int value = 0;
int i=0;

/*---------------------------------------------------------------------------*/
/* function prototypes                                                       */
/*---------------------------------------------------------------------------*/
void draw_array(char *plane1, char *plane2);
void wait(unsigned short);
void Draw2L(char *image1, char *image2);
void DrawSprite2L(int x, int y, char *src1, char *src2);
void EraseSprite2L(int x, int y, char *src1, char *src2);
void chk_key();
void draw_badgers();


/*===========================================================================*/
/* main routine: where all the fun starts ...                                */
/*===========================================================================*/
void _main(void)
{
    if (!GrayMode(GRAY_ON))
    {
        return;
    }
    FontSetSys(F_6x8);
		
    while(1)
    {
		//begin badger scene
		ClrScr();
		Draw2L(badger_bg_1,badger_bg_2);
		draw_badgers();		
		//end badger scene
		
		//begin mushroom scene
		ClrScr();
		Draw2L(mushroom1_1,mushroom1_2);
		DrawStr(50,90,"Mushroom!",A_REVERSE);
		chk_key();
		wait(500);
		Draw2L(mushroom2_1,mushroom2_2);
		DrawStr(50,90,"Mushroom!",A_REVERSE);
		chk_key();
		wait(1000);
		//end mushroom scene
    }
    
    ngetchx();
    GrayMode(GRAY_OFF);
}

void draw_badgers()		//recursive
{
	chk_key();
	if(i>11)
	{
		i=0;
		return ;
	}
	int a=0, j=0;
	for(a=0; a<6; a++)
	{
		Draw2L(badger_bg_1,badger_bg_2);
		for(j=0; j<=i; j++)
		{
			int x=badger_array[j][0];
			int y=badger_array[j][1];
			
			switch(a)
			{
			case 0:
				DrawSprite2L(x, y, body_up1, body_up2);	break;
			case 1:
				EraseSprite2L(x, y, body_up1, body_up2);	break;
			case 2:
				DrawSprite2L(x, y, body_mid1, body_mid2);	break;
			case 3:
				EraseSprite2L(x, y, body_mid1, body_mid2);	break;
			case 4:
				DrawSprite2L(x, y, body_down1, body_down2);	break;
			case 5:
				EraseSprite2L(x, y, body_down1, body_down2);	break;
			}
		}
		if(a%2 == 0)	//if state is even or draw mode, wait
		{
			wait(100);
		}	
	}
	i++;
	draw_badgers();
}

void chk_key()
{
	void *kbq = kbd_queue();
	unsigned short key;
	if (!OSdequeue (&key, kbq))
	{
 		 if(key == KEY_ESC)
 		 {
 				GrayMode(GRAY_OFF);
 		 		exit(0);
 		 }
     else if (key== '+' && value < 127)
        value++;
     else if (key== '-' && value > (TI89 ? -28 : 0))
        value--;
		 GrayAdjust(value);
	}
}
void EraseSprite2L(int x, int y, char *src1, char *src2)
{
	Sprite32(x, y, 32, src1, GetPlane(LIGHT_PLANE), SPRT_XOR);
	Sprite32(x, y, 32, src2, GetPlane(DARK_PLANE), SPRT_XOR);
}

void DrawSprite2L(int x, int y, char *src1, char *src2)
{
	Sprite32(x, y, 32, src1, GetPlane(LIGHT_PLANE), SPRT_OR);
	Sprite32(x, y, 32, src2, GetPlane(DARK_PLANE), SPRT_OR);
}

void Draw2L(char *image1, char *image2)
{
    unsigned char* plane1;
    unsigned char* plane2;
    unsigned char* dest1;
    unsigned char* dest2;
    unsigned char* src1;
    unsigned char* src2;
    int            loop;
	
    plane1 = GetPlane(0);
    plane2 = GetPlane(1);
	
    dest1 = plane1;
    dest2 = plane2;
    src1  = image1;
    src2  = image2;
	
    if (!TI89)
    {
        memset(plane1,0,LCD_SIZE);
        memset(plane2,0,LCD_SIZE);
        dest1 += ((128-100)*30)/2 + ((240-160)/8)/2;
        dest2 += ((128-100)*30)/2 + ((240-160)/8)/2;
    }
	
    for (loop=0;loop<100;loop++,src1+=20,src2+=20,dest1+=30,dest2+=30)
    {
        memcpy(dest1,src1,20);
        memcpy(dest2,src2,20);
    }
}